@@ -21,9 +21,10 @@ import sys
 from pathlib import Path
 from typing import Any, Sequence
 
+from griffe.agents.extensions import Extensions
+from griffe.agents.extensions.base import load_extensions
 from griffe.docstrings.parsers import Parser
 from griffe.encoders import Encoder
-from griffe.extensions import Extensions
 from griffe.loader import AsyncGriffeLoader, GriffeLoader
 from griffe.logger import get_logger
 
@@ -108,6 +109,13 @@ def get_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Whether to append sys.path to search paths specified with -s.",
     )
+    parser.add_argument(
+        "-e",
+        "--extensions",
+        default={},
+        type=json.loads,
+        help="A list of extensions to use.",
+    )
     parser.add_argument(
         "-f",
         "--full",
@@ -166,9 +174,9 @@ def main(args: list[str] | None = None) -> int:  # noqa: WPS231
         An exit code.
     """
     parser = get_parser()
-    opts: argparse.Namespace = parser.parse_args(args)  # type: ignore
+    opts: argparse.Namespace = parser.parse_args(args)
 
-    logging.basicConfig(format="%(levelname)-10s %(message)s", level=logging.WARNING)  # noqa: WPS323
+    logging.basicConfig(format="%(levelname)-10s %(message)s", level=logging.INFO)  # noqa: WPS323
 
     output = opts.output
 
@@ -180,7 +188,7 @@ def main(args: list[str] | None = None) -> int:  # noqa: WPS231
     if opts.append_sys_path:
         search.extend(sys.path)
 
-    extensions = Extensions()
+    extensions = load_extensions(opts.extensions)
 
     if opts.async_loader:
         loop = asyncio.get_event_loop()
