@@ -11,12 +11,12 @@ import inspect
 from functools import cached_property
 from pathlib import Path
 from textwrap import dedent
-from typing import Any, Callable
+from typing import Any, Callable, cast
 
 from griffe.collections import LinesCollection, ModulesCollection
 from griffe.docstrings.dataclasses import DocstringSection
 from griffe.docstrings.parsers import Parser, parse  # noqa: WPS347
-from griffe.exceptions import AliasResolutionError, NameResolutionError
+from griffe.exceptions import AliasResolutionError, BuiltinModuleError, NameResolutionError
 from griffe.expressions import Expression, Name
 from griffe.mixins import GetMembersMixin, ObjectAliasMixin, SetMembersMixin
 
@@ -86,9 +86,9 @@ class Docstring:
         self,
         value: str,
         *,
-        lineno: int | None,
-        endlineno: int | None,
-        parent: Module | Class | Function | Attribute | None = None,
+        lineno: int | None = None,
+        endlineno: int | None = None,
+        parent: Object | None = None,
         parser: Parser | None = None,
         parser_options: dict[str, Any] | None = None,
     ) -> None:
@@ -105,7 +105,7 @@ class Docstring:
         self.value: str = inspect.cleandoc(value)
         self.lineno: int | None = lineno
         self.endlineno: int | None = endlineno
-        self.parent: Module | Class | Function | Attribute | None = parent
+        self.parent: Object | None = parent
         self.parser: Parser | None = parser
         self.parser_options: dict[str, Any] = parser_options or {}
 
@@ -321,12 +321,13 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         self.labels: set[str] = set()
         self.imports: dict[str, str] = {}
         self.exports: set[str] | None = None
+        self.aliases: dict[str, Alias] = {}
         self._lines_collection: LinesCollection | None = lines_collection
         self._modules_collection: ModulesCollection | None = modules_collection
 
         # attach the docstring to this object
         if docstring:
-            docstring.parent = self  # type: ignore
+            docstring.parent = self
 
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__}({self.name!r}, {self.lineno!r}, {self.endlineno!r})>"
@@ -407,7 +408,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A dictionary of modules.
         """
-        return {name: member for name, member in self.members.items() if member.kind is Kind.MODULE}  # type: ignore
+        return {name: member for name, member in self.members.items() if member.kind is Kind.MODULE}  # type: ignore[misc]
 
     @property
     def classes(self) -> dict[str, Class]:
@@ -416,7 +417,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A dictionary of classes.
         """
-        return {name: member for name, member in self.members.items() if member.kind is Kind.CLASS}  # type: ignore
+        return {name: member for name, member in self.members.items() if member.kind is Kind.CLASS}  # type: ignore[misc]
 
     @property
     def functions(self) -> dict[str, Function]:
@@ -425,7 +426,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A dictionary of functions.
         """
-        return {name: member for name, member in self.members.items() if member.kind is Kind.FUNCTION}  # type: ignore
+        return {name: member for name, member in self.members.items() if member.kind is Kind.FUNCTION}  # type: ignore[misc]
 
     @property
     def attributes(self) -> dict[str, Attribute]:
@@ -434,7 +435,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A dictionary of attributes.
         """
-        return {name: member for name, member in self.members.items() if member.kind is Kind.ATTRIBUTE}  # type: ignore
+        return {name: member for name, member in self.members.items() if member.kind is Kind.ATTRIBUTE}  # type: ignore[misc]
 
     @cached_property
     def module(self) -> Module:
@@ -462,7 +463,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         """
         module = self.module
         while module.parent:
-            module = module.parent  # type: ignore
+            module = module.parent  # type: ignore[assignment]  # always a module
         return module
 
     @cached_property
@@ -486,7 +487,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A file path.
         """
-        return self.module.filepath.relative_to(self.package.filepath.parent.parent)  # type: ignore
+        return self.module.filepath.relative_to(self.package.filepath.parent.parent)
 
     @cached_property
     def path(self) -> str:
@@ -551,9 +552,13 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A list of lines.
         """
+        try:
+            filepath = self.filepath
+        except BuiltinModuleError:
+            return []
         if self.lineno is None or self.endlineno is None:
-            return self.lines_collection[self.filepath]
-        return self.lines_collection[self.filepath][self.lineno - 1 : self.endlineno]
+            return self.lines_collection[filepath]
+        return self.lines_collection[filepath][self.lineno - 1 : self.endlineno]
 
     @cached_property
     def source(self) -> str:
@@ -674,9 +679,11 @@ class Alias(ObjectAliasMixin):
         else:
             self._target = target
             self._target_path = target.path
+            if self.parent is not None:
+                target.aliases[self.path] = self
         self.lineno: int | None = lineno
         self.endlineno: int | None = endlineno
-        self.parent: Module | Class | None = parent
+        self._parent: Module | Class | None = parent
 
     def __getattr__(self, name: str) -> Any:
         # forward everything to the target
@@ -703,6 +710,21 @@ class Alias(ObjectAliasMixin):
         except AliasResolutionError:
             return Kind.ALIAS
 
+    @property
+    def parent(self) -> Module | Class | None:
+        """Return the parent of this alias.
+
+        Returns:
+            The parent.
+        """
+        return self._parent
+
+    @parent.setter
+    def parent(self, value: Module | Class) -> None:
+        self._parent = value
+        if self.resolved:
+            self._target.aliases[self.path] = self  # type: ignore[union-attr]  # we just checked the target is not None
+
     @cached_property
     def path(self) -> str:
         """Return the dotted path / import path of this object.
@@ -710,7 +732,7 @@ class Alias(ObjectAliasMixin):
         Returns:
             A dotted path.
         """
-        return ".".join((self.parent.path, self.name))  # type: ignore  # we assume there's always a parent
+        return ".".join((self.parent.path, self.name))  # type: ignore[union-attr]  # we assume there's always a parent
 
     @cached_property
     def modules_collection(self) -> ModulesCollection:
@@ -720,7 +742,7 @@ class Alias(ObjectAliasMixin):
             A modules collection.
         """
         # no need to forward to the target
-        return self.parent.modules_collection  # type: ignore  # we assume there's always a parent
+        return self.parent.modules_collection  # type: ignore[union-attr]  # we assume there's always a parent
 
     @property
     def target(self) -> Object | Alias:
@@ -734,7 +756,13 @@ class Alias(ObjectAliasMixin):
         """
         if not self.resolved:
             self.resolve_target()
-        return self._target  # type: ignore  # cannot return None, exception is raised
+        return self._target  # type: ignore[return-value]  # cannot return None, exception is raised
+
+    @target.setter
+    def target(self, value: Object | Alias) -> None:
+        self._target = value
+        if self.parent is not None:
+            self._target.aliases[self.path] = self
 
     def resolve_target(self) -> None:
         """Resolve the target.
@@ -750,6 +778,8 @@ class Alias(ObjectAliasMixin):
             self._target = self.modules_collection[self._target_path]
         except KeyError as error:
             raise AliasResolutionError(self._target_path) from error
+        if self.parent is not None:
+            self._target.aliases[self.path] = self  # type: ignore[union-attr]  # we just set the target
 
     @property
     def resolved(self) -> bool:
@@ -792,27 +822,35 @@ class Module(Object):
 
     kind = Kind.MODULE
 
-    def __init__(self, *args: Any, filepath: Path, **kwargs: Any) -> None:
+    def __init__(self, *args: Any, filepath: Path | None = None, **kwargs: Any) -> None:
         """Initialize the module.
 
         Parameters:
             *args: See [`griffe.dataclasses.Object`][].
-            filepath: The module file path. It can be null for namespace packages.
+            filepath: The module file path (directory for namespace [sub]packages, none for builtin modules).
             **kwargs: See [`griffe.dataclasses.Object`][].
         """
         super().__init__(*args, **kwargs)
-        self._filepath: Path = filepath
+        self._filepath: Path | None = filepath
 
     def __repr__(self) -> str:
-        return f"<Module({self._filepath!r})>"
+        try:
+            return f"<Module({self.filepath!r})>"
+        except BuiltinModuleError:
+            return f"<Module({self.name!r})>"
 
     @property
     def filepath(self) -> Path:
         """Get the file path of this module.
 
+        Raises:
+            BuiltinModuleError: When the instance filepath is None.
+
         Returns:
             The module's file path.
         """
+        if self._filepath is None:
+            raise BuiltinModuleError(self.name)
         return self._filepath
 
     @cached_property
@@ -822,7 +860,10 @@ class Module(Object):
         Returns:
             True or False.
         """
-        return self.filepath.name == "__init__.py"  # type: ignore
+        try:
+            return self.filepath.name == "__init__.py"
+        except BuiltinModuleError:
+            return False
 
     @cached_property
     def is_package(self) -> bool:
@@ -849,7 +890,10 @@ class Module(Object):
         Returns:
             True or False.
         """
-        return not self.parent and self.filepath.is_dir()
+        try:
+            return not self.parent and self.filepath.is_dir()
+        except BuiltinModuleError:
+            return False
 
     @cached_property
     def is_namespace_subpackage(self) -> bool:
@@ -858,14 +902,18 @@ class Module(Object):
         Returns:
             True or False.
         """
-        return (
-            self.parent
-            and self.filepath.is_dir()
-            and self.parent.is_namespace_package  # type: ignore  # modules parents are always modules
-            or self.parent.is_namespace_subpackage  # type: ignore  # modules parents are always modules
-        )
+        try:
+            return (
+                self.parent is not None
+                and self.filepath.is_dir()
+                and (
+                    cast(Module, self.parent).is_namespace_package or cast(Module, self.parent).is_namespace_subpackage
+                )
+            )
+        except BuiltinModuleError:
+            return False
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore[override]
         """Return this module's data as a dictionary.
 
         Parameters:
@@ -875,7 +923,7 @@ class Module(Object):
             A dictionary.
         """
         base = super().as_dict(**kwargs)
-        base["filepath"] = str(self.filepath) if self.filepath else None
+        base["filepath"] = str(self._filepath) if self._filepath else None
         return base
 
 
@@ -903,7 +951,7 @@ class Class(Object):
         self.bases: list[Name | Expression] = bases or []
         self.decorators: list[Decorator] = decorators or []
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore[override]
         """Return this class' data as a dictionary.
 
         Parameters:
@@ -945,7 +993,7 @@ class Function(Object):
         self.returns: str | Name | Expression | None = returns
         self.decorators: list[Decorator] = decorators or []
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore[override]
         """Return this function's data as a dictionary.
 
         Parameters:
@@ -985,7 +1033,7 @@ class Attribute(Object):
         self.value: str | None = value
         self.annotation: str | Name | Expression | None = annotation
 
-    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore
+    def as_dict(self, **kwargs: Any) -> dict[str, Any]:  # type: ignore[override]
         """Return this function's data as a dictionary.
 
         Parameters:
