@@ -7,11 +7,12 @@ from ast import PyCF_ONLY_AST
 from contextlib import suppress
 from typing import TYPE_CHECKING, Any, List, Pattern, Tuple
 
+from griffe.agents.nodes import get_annotation
 from griffe.docstrings.dataclasses import (
     DocstringAdmonition,
     DocstringAttribute,
-    DocstringException,
     DocstringParameter,
+    DocstringRaise,
     DocstringReceive,
     DocstringReturn,
     DocstringSection,
@@ -21,7 +22,6 @@ from griffe.docstrings.dataclasses import (
 )
 from griffe.docstrings.utils import warning
 from griffe.expressions import Expression, Name
-from griffe.node_utils import get_annotation
 
 if TYPE_CHECKING:
     from griffe.dataclasses import Docstring
@@ -52,6 +52,9 @@ ItemsBlock = Tuple[BlockItems, int]
 _RE_ADMONITION: Pattern = re.compile(r"^(?P<type>[\w][\s\w-]*):(\s+(?P<title>.+))?$", re.I)
 """Regular expression to match admonitions, of the form `TYPE: [TITLE]`."""
 
+_RE_NAME_ANNOTATION_DESCRIPTION: Pattern = re.compile(r"^(?:(?P<name>\w+)?\s*(?:\((?P<type>.+)\))?:\s*)?(?P<desc>.*)$")
+"""Regular expression to match `name (type): Description` in docstrings sections items."""
+
 
 def _read_block_items(docstring: Docstring, offset: int) -> ItemsBlock:  # noqa: WPS231
     lines = docstring.lines
@@ -168,22 +171,22 @@ def _read_parameters(docstring: Docstring, offset: int) -> tuple[list[DocstringP
         if " " in name_with_type:
             name, annotation = name_with_type.split(" ", 1)
             annotation = annotation.strip("()")
-            if annotation.endswith(", optional"):  # type: ignore
-                annotation = annotation[:-10]  # type: ignore
+            if annotation.endswith(", optional"):
+                annotation = annotation[:-10]
             # try to compile the annotation to transform it into an expression
             with suppress(SyntaxError, AttributeError):
                 code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
-                annotation = get_annotation(code.body, parent=docstring.parent)  # type: ignore
+                annotation = code.body and get_annotation(code.body, parent=docstring.parent)  # type: ignore[arg-type]
         else:
             name = name_with_type
             # try to use the annotation from the signature
             try:
-                annotation = docstring.parent.parameters[name].annotation  # type: ignore
+                annotation = docstring.parent.parameters[name].annotation  # type: ignore[union-attr]
             except (AttributeError, KeyError):
                 annotation = None
 
         try:
-            default = docstring.parent.parameters[name].default  # type: ignore
+            default = docstring.parent.parameters[name].default  # type: ignore[union-attr]
         except (AttributeError, KeyError):
             default = None
 
@@ -237,11 +240,11 @@ def _read_attributes_section(docstring: Docstring, offset: int) -> tuple[Docstri
             # try to compile the annotation to transform it into an expression
             with suppress(SyntaxError):
                 code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
-                annotation = get_annotation(code.body, parent=docstring.parent)  # type: ignore
+                annotation = code.body and get_annotation(code.body, parent=docstring.parent)  # type: ignore[arg-type]
         else:
             name = name_with_type
             try:
-                annotation = docstring.parent.attributes[name].annotation  # type: ignore
+                annotation = docstring.parent.attributes[name].annotation  # type: ignore[union-attr]
             except (AttributeError, KeyError):
                 annotation = None
 
@@ -265,7 +268,7 @@ def _read_raises_section(docstring: Docstring, offset: int) -> tuple[DocstringSe
             _warn(docstring, line_number, f"Failed to get 'exception: description' pair from '{exception_lines[0]}'")
         else:
             description = "\n".join([description.lstrip(), *exception_lines[1:]]).rstrip("\n")
-            exceptions.append(DocstringException(annotation=annotation, description=description))
+            exceptions.append(DocstringRaise(annotation=annotation, description=description))
 
     if exceptions:
         return DocstringSection(DocstringSectionKind.raises, exceptions), new_offset
@@ -294,105 +297,123 @@ def _read_warns_section(docstring: Docstring, offset: int) -> tuple[DocstringSec
     return None, new_offset
 
 
-def _read_returns_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
-    text, new_offset = _read_block(docstring, offset)
+def _read_returns_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+    returns = []
+    block, new_offset = _read_block_items(docstring, offset)
 
-    # early exit if there is no text in the return section
-    if not text:
-        _warn(docstring, new_offset, f"Empty return section at line {offset}")
-        return None, new_offset
+    for index, (line_number, return_lines) in enumerate(block):
+        match = _RE_NAME_ANNOTATION_DESCRIPTION.match(return_lines[0])
+        if not match:
+            _warn(docstring, line_number, f"Failed to get name, annotation or description from '{return_lines[0]}'")
+            continue
 
-    # check the presence of a name and description, separated by a semi-colon
-    try:
-        type_, text = text.split(":", 1)
-    except ValueError:
-        description = text
-        # try to use the annotation from the signature
-        try:  # noqa: WPS505
-            annotation = docstring.parent.returns  # type: ignore
-        except AttributeError:
-            annotation = None
-    else:
-        annotation = type_.lstrip()
-        description = text.lstrip()
+        name, annotation, description = match.groups()
+        description = "\n".join([description.lstrip(), *return_lines[1:]]).rstrip("\n")
 
-    # there was no type in the docstring and no return annotation in the signature
-    if annotation is None:
-        _warn(docstring, new_offset, "No return type/annotation in docstring/signature")
+        if annotation and docstring.parent is not None:
+            # try to compile the annotation to transform it into an expression
+            with suppress(SyntaxError):
+                code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
+                annotation = code.body and get_annotation(code.body, parent=docstring.parent)  # type: ignore[arg-type]
+        else:
+            # try to retrieve the annotation from the docstring parent
+            with suppress(AttributeError, KeyError):
+                annotation = docstring.parent.returns  # type: ignore[union-attr]
+                if len(block) > 1:
+                    if annotation.is_tuple:
+                        annotation = annotation.tuple_item(index)
 
-    return (
-        DocstringSection(DocstringSectionKind.returns, DocstringReturn(annotation=annotation, description=description)),
-        new_offset,
-    )
+            if annotation is None:
+                returned_value = repr(name) or index
+                _warn(docstring, line_number, f"No type or annotation for returned value {returned_value}")
+
+        returns.append(DocstringReturn(name=name or "", annotation=annotation, description=description))
+
+    if returns:
+        return DocstringSection(DocstringSectionKind.returns, returns), new_offset
+
+    _warn(docstring, new_offset, f"Empty returns section at line {offset}")
+    return None, new_offset
 
 
-def _read_yields_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
-    text, new_offset = _read_block(docstring, offset)
+def _read_yields_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+    yields = []
+    block, new_offset = _read_block_items(docstring, offset)
 
-    # early exit if there is no text in the yield section
-    if not text:
-        _warn(docstring, new_offset, f"Empty yield section at line {offset}")
-        return None, new_offset
+    for index, (line_number, yield_lines) in enumerate(block):  # noqa: B007 (will be used later)
+        match = _RE_NAME_ANNOTATION_DESCRIPTION.match(yield_lines[0])
+        if not match:
+            _warn(docstring, line_number, f"Failed to get name, annotation or description from '{yield_lines[0]}'")
+            continue
 
-    # check the presence of a name and description, separated by a semi-colon
-    try:
-        type_, text = text.split(":", 1)
-    except ValueError:
-        description = text
-        # try to use the annotation from the signature
-        try:  # noqa: WPS505
-            # TODO: handle Iterator and Generator types
-            annotation = docstring.parent.returns  # type: ignore
-        except AttributeError:
-            annotation = None
-    else:
-        annotation = type_.lstrip()
-        description = text.lstrip()
+        name, annotation, description = match.groups()
+        description = "\n".join([description.lstrip(), *yield_lines[1:]]).rstrip("\n")
 
-    # there was no type in the docstring and no return annotation in the signature
-    if annotation is None:
-        _warn(docstring, new_offset, "No yield type/annotation in docstring/signature")
+        if annotation and docstring.parent is not None:
+            # try to compile the annotation to transform it into an expression
+            with suppress(SyntaxError):
+                code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
+                annotation = code.body and get_annotation(code.body, parent=docstring.parent)  # type: ignore[arg-type]
+        else:
+            # try to retrieve the annotation from the docstring parent
+            with suppress(AttributeError, KeyError):
+                annotation = docstring.parent.returns  # type: ignore[union-attr]
+                # TODO: support getting yield part and exploding tuple (in a generator/iterator)
+                # if len(block) > 1:
+                #     if annotation.is_tuple:
+                #         annotation = annotation.tuple_item(index)
 
-    return (
-        DocstringSection(DocstringSectionKind.yields, DocstringYield(annotation=annotation, description=description)),
-        new_offset,
-    )
+            if annotation is None:
+                yielded_value = repr(name) or index
+                _warn(docstring, line_number, f"No type or annotation for yielded value {yielded_value}")
+
+        yields.append(DocstringYield(name=name or "", annotation=annotation, description=description))
+
+    if yields:
+        return DocstringSection(DocstringSectionKind.yields, yields), new_offset
+
+    _warn(docstring, new_offset, f"Empty yields section at line {offset}")
+    return None, new_offset
 
 
-def _read_receives_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:
-    text, new_offset = _read_block(docstring, offset)
+def _read_receives_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
+    receives = []
+    block, new_offset = _read_block_items(docstring, offset)
 
-    # early exit if there is no text in the receive section
-    if not text:
-        _warn(docstring, new_offset, f"Empty receives section at line {offset}")
-        return None, new_offset
+    for index, (line_number, receive_lines) in enumerate(block):  # noqa: B007 (will be used later)
+        match = _RE_NAME_ANNOTATION_DESCRIPTION.match(receive_lines[0])
+        if not match:
+            _warn(docstring, line_number, f"Failed to get name, annotation or description from '{receive_lines[0]}'")
+            continue
 
-    # check the presence of a name and description, separated by a semi-colon
-    try:
-        type_, text = text.split(":", 1)
-    except ValueError:
-        description = text
-        # try to use the annotation from the signature
-        try:  # noqa: WPS505
-            # TODO: handle Iterator and Generator types
-            annotation = docstring.parent.returns  # type: ignore
-        except AttributeError:
-            annotation = None
-    else:
-        annotation = type_.lstrip()
-        description = text.lstrip()
+        name, annotation, description = match.groups()
+        description = "\n".join([description.lstrip(), *receive_lines[1:]]).rstrip("\n")
 
-    # there was no type in the docstring and no return annotation in the signature
-    if annotation is None:
-        _warn(docstring, new_offset, "No receive type/annotation in docstring/signature")
+        if annotation and docstring.parent is not None:
+            # try to compile the annotation to transform it into an expression
+            with suppress(SyntaxError):
+                code = compile(annotation, mode="eval", filename="", flags=PyCF_ONLY_AST, optimize=2)
+                annotation = code.body and get_annotation(code.body, parent=docstring.parent)  # type: ignore[arg-type]
+        # else:
+        # try to retrieve the annotation from the docstring parent
+        # TODO: support getting receive part and exploding tuple (in a generator/iterator)
+        # with suppress(AttributeError, KeyError):
+        #     annotation = docstring.parent.returns  # type: ignore[union-attr]
+        #     if len(block) > 1:
+        #         if annotation.is_tuple:
+        #             annotation = annotation.tuple_item(index)
 
-    return (
-        DocstringSection(
-            DocstringSectionKind.receives,
-            DocstringReceive(annotation=annotation, description=description),
-        ),
-        new_offset,
-    )
+        if annotation is None:
+            received_value = repr(name) or index
+            _warn(docstring, line_number, f"No type or annotation for received value {received_value}")
+
+        receives.append(DocstringReceive(name=name or "", annotation=annotation, description=description))
+
+    if receives:
+        return DocstringSection(DocstringSectionKind.receives, receives), new_offset
+
+    _warn(docstring, new_offset, f"Empty receives section at line {offset}")
+    return None, new_offset
 
 
 def _read_examples_section(docstring: Docstring, offset: int) -> tuple[DocstringSection | None, int]:  # noqa: WPS231
@@ -554,8 +575,8 @@ def parse(  # noqa: WPS231
                         )
                     )
                 else:
-                    offset -= 1
-                    current_section.append(lines[offset])
+                    with suppress(IndexError):
+                        current_section.append(lines[offset])
         else:
             current_section.append(lines[offset])
 
