@@ -18,14 +18,14 @@ from contextlib import suppress
 from pathlib import Path
 from typing import Any, Iterator, Sequence, Tuple
 
+from griffe.agents.extensions import Extensions
+from griffe.agents.inspector import inspect
+from griffe.agents.visitor import patch_ast, visit
 from griffe.collections import LinesCollection, ModulesCollection
 from griffe.dataclasses import Module, Object
 from griffe.docstrings.parsers import Parser
-from griffe.exceptions import AliasResolutionError, UnhandledPthFileError
-from griffe.extended_ast import extend_ast
-from griffe.extensions import Extensions
+from griffe.exceptions import AliasResolutionError, UnhandledPthFileError, UnimportableModuleError
 from griffe.logger import get_logger
-from griffe.visitor import visit
 
 NamePartsType = Tuple[str, ...]
 NamePartsAndPathType = Tuple[NamePartsType, Path]
@@ -38,19 +38,22 @@ async def _read_async(path):
         return await fd.read()
 
 
-async def _fallback_read_async(path):
-    logger.warning("aiofiles is not installed, fallback to blocking read")
+async def _read_sync(path):
     return path.read_text()
 
 
 try:
-    from aiofiles import open as aopen  # type: ignore
+    from aiofiles import open as aopen
 except ModuleNotFoundError:
-    read_async = _fallback_read_async
+    logger.warning("aiofiles is not installed, fallback to blocking read")
+    read_async = _read_sync
 else:
     read_async = _read_async
 
 
+_builtin_modules: set[str] = set(sys.builtin_module_names)
+
+
 class _BaseGriffeLoader:
     def __init__(
         self,
@@ -74,32 +77,60 @@ class _BaseGriffeLoader:
         self.docstring_options: dict[str, Any] = docstring_options or {}
         self.lines_collection: LinesCollection = lines_collection or LinesCollection()
         self.modules_collection: ModulesCollection = modules_collection or ModulesCollection()
-        extend_ast()
+        patch_ast()
 
-    def _module_name_and_path(
-        self,
-        module: str | Path,
-        search_paths: Sequence[str | Path] | None = None,
-    ) -> tuple[str, Path]:
-        if isinstance(module, Path):
-            # programatically passed a Path, try only that
-            module_name, module_path = module_name_path(module)
-        else:
-            # passed a string (from CLI or Python code), try both
-            try:
-                module_name, module_path = module_name_path(Path(module))
-            except FileNotFoundError:
-                module_name = module
-                module_path = find_module(module_name, search_paths=search_paths)
-        return module_name, module_path
+    def _create_module(self, module_name: str, module_path: Path) -> Module:
+        return Module(
+            module_name,
+            filepath=module_path,
+            lines_collection=self.lines_collection,
+            modules_collection=self.modules_collection,
+        )
+
+    def _visit_module(self, code: str, module_name: str, module_path: Path, parent: Module | None = None) -> Module:
+        self.lines_collection[module_path] = code.splitlines(keepends=False)
+        return visit(
+            module_name,
+            filepath=module_path,
+            code=code,
+            extensions=self.extensions,
+            parent=parent,
+            docstring_parser=self.docstring_parser,
+            docstring_options=self.docstring_options,
+            lines_collection=self.lines_collection,
+            modules_collection=self.modules_collection,
+        )
+
+    def _inspect_module(self, module_name: str, filepath: Path | None = None, parent: Module | None = None) -> Module:
+        return inspect(
+            module_name,
+            filepath=filepath,
+            extensions=self.extensions,
+            parent=parent,
+            docstring_parser=self.docstring_parser,
+            docstring_options=self.docstring_options,
+            lines_collection=self.lines_collection,
+        )
+
+    def _member_parent(self, module: Module, subparts: NamePartsType, subpath: Path) -> Module:
+        parent_parts = subparts[:-1]
+        try:
+            return module[parent_parts]
+        except KeyError:
+            if module.is_namespace_package or module.is_namespace_subpackage:
+                member_parent = Module(
+                    subparts[0],
+                    filepath=subpath.parent,
+                    lines_collection=self.lines_collection,
+                    modules_collection=self.modules_collection,
+                )
+                module[parent_parts] = member_parent
+                return member_parent
+        raise UnimportableModuleError(f"{subpath} is not importable")
 
 
 class GriffeLoader(_BaseGriffeLoader):
-    """The Griffe loader, allowing to load data from modules.
-
-    Attributes:
-        extensions: The extensions to use.
-    """
+    """The Griffe loader, allowing to load data from modules."""
 
     def load_module(
         self,
@@ -117,10 +148,14 @@ class GriffeLoader(_BaseGriffeLoader):
         Returns:
             A module.
         """
-        module_name, module_path = self._module_name_and_path(module, search_paths)
-        module_object = self._load_module_path(module_name, module_path, submodules=submodules)
-        self.modules_collection[module_object.path] = module_object
-        return module_object
+        if module in _builtin_modules:
+            module_name = module
+            top_module = self._inspect_module(module)  # type: ignore[arg-type]
+        else:
+            module_name, top_module_name, top_module_path = _top_name_and_path(module, search_paths)
+            top_module = self._load_module_path(top_module_name, top_module_path, submodules=submodules)
+        self.modules_collection[top_module.path] = top_module
+        return self.modules_collection[module_name]  # type: ignore[index]
 
     def follow_aliases(self, obj: Object, only_exported: bool = True) -> bool:  # noqa: WPS231
         """Follow aliases: try to recursively resolve all found aliases.
@@ -138,7 +173,7 @@ class GriffeLoader(_BaseGriffeLoader):
                 if only_exported and not obj.member_is_exported(member, explicitely=True):
                     continue
                 try:
-                    member.resolve_target()  # type: ignore  # we know it's an alias
+                    member.resolve_target()  # type: ignore[union-attr]  # we know it's an alias
                 except AliasResolutionError as error:
                     success = False
                     package = error.target_path.split(".", 1)[0]
@@ -146,7 +181,7 @@ class GriffeLoader(_BaseGriffeLoader):
                         with suppress(ModuleNotFoundError):
                             self.load_module(package)
             else:
-                success &= self.follow_aliases(member)  # type: ignore  # we know it's an object
+                success &= self.follow_aliases(member)  # type: ignore[arg-type]  # we know it's an object
         return success
 
     def _load_module_path(
@@ -157,18 +192,12 @@ class GriffeLoader(_BaseGriffeLoader):
         parent: Module | None = None,
     ) -> Module:
         logger.debug(f"Loading path {module_path}")
-        code = module_path.read_text()
-        self.lines_collection[module_path] = code.splitlines(keepends=False)
-        module = visit(
-            module_name,
-            filepath=module_path,
-            code=code,
-            extensions=self.extensions,
-            parent=parent,
-            docstring_parser=self.docstring_parser,
-            docstring_options=self.docstring_options,
-            lines_collection=self.lines_collection,
-        )
+        try:
+            code = module_path.read_text()
+        except OSError:
+            module = self._create_module(module_name, module_path)
+        else:
+            module = self._visit_module(code, module_name, module_path, parent)
         if submodules:
             self._load_submodules(module)
         return module
@@ -178,11 +207,10 @@ class GriffeLoader(_BaseGriffeLoader):
             self._load_submodule(module, subparts, subpath)
 
     def _load_submodule(self, module: Module, subparts: NamePartsType, subpath: Path) -> None:
-        parent_parts = subparts[:-1]
         try:
-            member_parent = module[parent_parts]
-        except KeyError:
-            logger.debug(f"Skipping (not importable) {subpath}")
+            member_parent = self._member_parent(module, subparts, subpath)
+        except UnimportableModuleError as error:
+            logger.warning(f"{error}. Missing __init__ module?")
         else:
             member_parent[subparts[-1]] = self._load_module_path(
                 subparts[-1], subpath, submodules=False, parent=member_parent
@@ -190,11 +218,7 @@ class GriffeLoader(_BaseGriffeLoader):
 
 
 class AsyncGriffeLoader(_BaseGriffeLoader):
-    """The asynchronous Griffe loader, allowing to load data from modules.
-
-    Attributes:
-        extensions: The extensions to use.
-    """
+    """The asynchronous Griffe loader, allowing to load data from modules."""
 
     async def load_module(
         self,
@@ -212,10 +236,10 @@ class AsyncGriffeLoader(_BaseGriffeLoader):
         Returns:
             A module.
         """
-        module_name, module_path = self._module_name_and_path(module, search_paths)
-        module_object = await self._load_module_path(module_name, module_path, submodules=submodules)
-        self.modules_collection[module_object.path] = module_object
-        return module_object
+        module_name, top_module_name, top_module_path = _top_name_and_path(module, search_paths)
+        top_module = await self._load_module_path(top_module_name, top_module_path, submodules=submodules)
+        self.modules_collection[top_module.path] = top_module
+        return self.modules_collection[module_name]
 
     async def follow_aliases(self, obj: Object, only_exported: bool = True) -> bool:  # noqa: WPS231
         """Follow aliases: try to recursively resolve all found aliases.
@@ -233,7 +257,7 @@ class AsyncGriffeLoader(_BaseGriffeLoader):
                 if only_exported and not obj.member_is_exported(member, explicitely=True):
                     continue
                 try:
-                    member.resolve_target()  # type: ignore  # we know it's an alias
+                    member.resolve_target()  # type: ignore[union-attr]  # we know it's an alias
                 except AliasResolutionError as error:
                     success = False
                     package = error.target_path.split(".", 1)[0]
@@ -241,7 +265,7 @@ class AsyncGriffeLoader(_BaseGriffeLoader):
                         with suppress(ModuleNotFoundError):
                             await self.load_module(package)
             else:
-                success &= self.follow_aliases(member)  # type: ignore  # we know it's an object
+                success &= await self.follow_aliases(member)  # type: ignore[arg-type]  # we know it's an object
         return success
 
     async def _load_module_path(
@@ -252,18 +276,12 @@ class AsyncGriffeLoader(_BaseGriffeLoader):
         parent: Module | None = None,
     ) -> Module:
         logger.debug(f"Loading path {module_path}")
-        code = await read_async(module_path)
-        self.lines_collection[module_path] = code.splitlines(keepends=False)
-        module = visit(
-            module_name,
-            filepath=module_path,
-            code=code,
-            extensions=self.extensions,
-            parent=parent,
-            docstring_parser=self.docstring_parser,
-            docstring_options=self.docstring_options,
-            lines_collection=self.lines_collection,
-        )
+        try:
+            code = await read_async(module_path)
+        except OSError:
+            module = self._create_module(module_name, module_path)
+        else:
+            module = self._visit_module(code, module_name, module_path, parent)
         if submodules:
             await self._load_submodules(module)
         return module
@@ -277,37 +295,74 @@ class AsyncGriffeLoader(_BaseGriffeLoader):
         )
 
     async def _load_submodule(self, module: Module, subparts: NamePartsType, subpath: Path) -> None:
-        parent_parts = subparts[:-1]
         try:
-            member_parent = module[parent_parts]
-        except KeyError:
-            logger.debug(f"Skipping (not importable) {subpath}")
+            member_parent = self._member_parent(module, subparts, subpath)
+        except UnimportableModuleError as error:
+            logger.debug(str(error))
         else:
             member_parent[subparts[-1]] = await self._load_module_path(
                 subparts[-1], subpath, submodules=False, parent=member_parent
             )
 
 
-def _module_depth(name_parts_and_path: NamePartsAndPathType) -> int:
-    return len(name_parts_and_path[0])
+def _top_name_and_path(
+    module: str | Path,
+    search_paths: Sequence[str | Path] | None = None,
+) -> tuple[str, str, Path]:
+    module_name, module_path = find_module_or_path(module, search_paths)
+    module_parts = module_name.split(".")
+    top_module_name = module_parts[0]
+    top_module_path = module_path
+    for _ in range(len(module_parts) - 1):
+        top_module_path = top_module_path.parent
+    return module_name, top_module_name, top_module_path
 
 
-def module_name_path(path: Path) -> tuple[str, Path]:
-    """Get the module name and path from a path.
+def find_module_or_path(
+    module: str | Path,
+    search_paths: Sequence[str | Path] | None = None,
+) -> tuple[str, Path]:
+    """Find the name and path of a module.
+
+    If a Path is passed, only try to find the module as a file path.
+    If a string is passed, first try to find the module as a file path,
+    then look into the search paths.
 
     Parameters:
-        path: A directory or file path. Paths to `__init__.py` files
-            will be resolved to their parent directory.
+        module: The module name or path.
+        search_paths: The paths to search into.
 
     Raises:
-        FileNotFoundError: When:
+        FileNotFoundError: When a Path was passed and the module could not be found:
 
             - the directory has no `__init__.py` file in it
             - the path does not exist
 
+        ModuleNotFoundError: When a string was passed and the module could not be found:
+
+            - no `module/__init__.py`
+            - no `module.py`
+            - no `module.pth`
+            - no `module` directory (namespace packages)
+            - or unsupported .pth file
+
     Returns:
         The name of the module (or package) and its path.
     """
+    if isinstance(module, Path):
+        # programatically passed a Path, try only that
+        module_name, module_path = _module_name_path(module)
+    else:
+        # passed a string (from CLI or Python code), try both
+        try:
+            module_name, module_path = _module_name_path(Path(module))
+        except FileNotFoundError:
+            module_name = module
+            module_path = find_module(module_name, search_paths=search_paths)
+    return module_name, module_path
+
+
+def _module_name_path(path: Path) -> tuple[str, Path]:
     if path.is_dir():
         module_path = path / "__init__.py"
         if module_path.exists():
@@ -341,14 +396,30 @@ def find_module(module_name: str, search_paths: Sequence[str | Path] | None = No
     search = [path if isinstance(path, Path) else Path(path) for path in search_paths or sys.path]
     parts = module_name.split(".")
 
-    filenames = [
+    # always search a .pth file first using the first part
+    for path in search:
+        top_pth = Path(f"{parts[0]}.pth")
+        abs_top_pth = path / top_pth
+        if abs_top_pth.exists():
+            with suppress(UnhandledPthFileError):
+                location = _handle_pth_file(abs_top_pth)
+                if location.suffix == ".py":
+                    location = location.parent
+                search = [location.parent]
+                # TODO: possible optimization
+                # always break if exists?
+                break
+
+    # resume regular search
+    filepaths = [
         Path(*parts, "__init__.py"),
         Path(*parts[:-1], f"{parts[-1]}.py"),
         Path(*parts[:-1], f"{parts[-1]}.pth"),
+        Path(*parts),  # namespace packages, try last
     ]
 
-    for path in search:
-        for choice in filenames:
+    for path in search:  # noqa: WPS440
+        for choice in filepaths:
             abs_path = path / choice
             # optimization: just check if the file exists,
             # not if it's an actual file
@@ -364,11 +435,18 @@ def find_module(module_name: str, search_paths: Sequence[str | Path] | None = No
 
 
 def _handle_pth_file(path):
-    instructions = path.read_text().split(";")
     # support for .pth files pointing to a directory
-    new_path = Path(instructions[0]) / "__init__.py"
-    if new_path.exists():
-        return new_path
+    instructions = path.read_text().strip("\n").split(";")
+
+    filepaths = [
+        Path(instructions[0], path.stem, "__init__.py"),
+        Path(instructions[0], path.stem),  # namespace packages, try last
+    ]
+
+    for choice in filepaths:
+        if choice.exists():
+            return choice
+
     # support for .pth files written by PDM, using editables
     module_name = path.stem
     if instructions[0] == f"import _{module_name}":
@@ -378,6 +456,7 @@ def _handle_pth_file(path):
         new_path = Path(editables_lines[-1].split("'")[3])
         if new_path.exists():
             return new_path
+
     raise UnhandledPthFileError(path)
 
 
@@ -388,8 +467,8 @@ def iter_submodules(path: Path) -> Iterator[NamePartsAndPathType]:  # noqa: WPS2
         path: The module path.
 
     Yields:
-        This generator yields tuples containing the parts
-        of the submodule name as well as its filepath.
+        name_parts: The parts of a submodule name.
+        filepath: A submodule filepath.
     """
     if path.name == "__init__.py":
         path = path.parent
@@ -411,3 +490,7 @@ def iter_submodules(path: Path) -> Iterator[NamePartsAndPathType]:  # noqa: WPS2
             yield rel_subpath.parts[:-1], subpath
         else:
             yield rel_subpath.with_suffix("").parts, subpath
+
+
+def _module_depth(name_parts_and_path: NamePartsAndPathType) -> int:
+    return len(name_parts_and_path[0])
