@@ -22,8 +22,20 @@ class GetMembersMixin:
         if not parts:
             return self
         if len(parts) == 1:
-            return self.members[parts[0]]  # type: ignore
-        return self.members[parts[0]][parts[1]]  # type: ignore
+            return self.members[parts[0]]  # type: ignore[attr-defined]
+        return self.members[parts[0]][parts[1]]  # type: ignore[attr-defined]
+
+
+def _get_parts(key: str | Sequence[str]) -> Sequence[str]:
+    if isinstance(key, str):
+        if not key:
+            raise ValueError("cannot set self (empty key)")
+        parts = key.split(".", 1)
+    else:
+        parts = list(key)
+    if not parts:
+        raise ValueError("cannot set self (empty parts)")
+    return parts
 
 
 class SetMembersMixin:
@@ -35,19 +47,16 @@ class SetMembersMixin:
     """
 
     def __setitem__(self, key: str | Sequence[str], value):
-        if isinstance(key, str):
-            if not key:
-                raise ValueError("cannot set self (empty key)")
-            parts = key.split(".", 1)
-        else:
-            parts = list(key)
-        if not parts:
-            raise ValueError("cannot set self (empty parts)")
+        parts = _get_parts(key)
         if len(parts) == 1:
-            self.members[parts[0]] = value  # type: ignore
+            name = parts[0]
+            if name in self.members:  # type: ignore[attr-defined]
+                for alias in self.members[name].aliases.values():  # type: ignore[attr-defined]
+                    alias.target = value
+            self.members[name] = value  # type: ignore[attr-defined]
             value.parent = self
         else:
-            self.members[parts[0]][parts[1]] = value  # type: ignore
+            self.members[parts[0]][parts[1]] = value  # type: ignore[attr-defined]
 
 
 class SetCollectionMembersMixin:
@@ -59,19 +68,16 @@ class SetCollectionMembersMixin:
     """
 
     def __setitem__(self, key: str | Sequence[str], value):
-        if isinstance(key, str):
-            if not key:
-                raise ValueError("cannot set self (empty key)")
-            parts = key.split(".", 1)
-        else:
-            parts = list(key)
-        if not parts:
-            raise ValueError("cannot set self (empty parts)")
+        parts = _get_parts(key)
         if len(parts) == 1:
-            self.members[parts[0]] = value  # type: ignore
+            name = parts[0]
+            if name in self.members:  # type: ignore[attr-defined]
+                for alias in self.members[name].aliases.values():  # type: ignore[attr-defined]
+                    alias.target = value
+            self.members[name] = value  # type: ignore[attr-defined]
             value._modules_collection = self  # noqa: WPS437
         else:
-            self.members[parts[0]][parts[1]] = value  # type: ignore
+            self.members[parts[0]][parts[1]] = value  # type: ignore[attr-defined]
 
 
 class ObjectAliasMixin:
@@ -86,7 +92,7 @@ class ObjectAliasMixin:
         Returns:
             True or False.
         """
-        return self.parent.member_is_exported(self, explicitely=explicitely)  # type: ignore
+        return self.parent.member_is_exported(self, explicitely=explicitely)  # type: ignore[attr-defined]
 
     @property
     def is_explicitely_exported(self) -> bool:
@@ -104,4 +110,4 @@ class ObjectAliasMixin:
         Returns:
             True or False.
         """
-        return self.parent.exports is None  # type: ignore
+        return self.parent.exports is None  # type: ignore[attr-defined]
